/*
    ...... - a free ... implementation
    Copyright (C) 1999-1999, Robert Chou <rob_chou@bigfoot.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

package org.gjt.jsp;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;


/**
 * 
 *	Pretends to be a normal HttpServletResponse to a servlet,
 *	except it caches all output data from the servlet.
 *
 *	All headers are passed through to the real response object.
 *	This class also keeps track of any header calls which are not
 *	suitable for caching.
 *
 */
public class CacheResponse implements HttpServletResponse
{
    HttpServletResponse	m_response;

	CacheOutputStream	m_outputStream;
    PrintWriter			m_printWriter;
	boolean				m_bGotOutput = false;

	boolean				m_bSentContentLength = false;
	boolean				m_bCantCache = false;
	int					m_iStatus = HttpServletResponse.SC_OK;

    CacheResponse( HttpServletResponse response )
	{
		m_response = response;

		m_outputStream = new CacheOutputStream();
    }

	// our methods
	public boolean getCantCache()
	{
		return m_bCantCache;
	}

	public void sendCachedResponse()
	throws IOException
	{
		OutputStream	realStream = m_response.getOutputStream();

		m_outputStream.writeTo( realStream );
		realStream.close();
	}

	public void saveCachedResponse( String strFilePath, long lRecycle )
	throws IOException
	{
		long	lCurrent = System.currentTimeMillis();

		m_response.setDateHeader( "Last-Modified", lCurrent );

		if( m_bSentContentLength == false )
		{
			m_bSentContentLength = true;

			m_response.setContentLength( m_outputStream.size() );
		}

		if( lRecycle > 0 )
		{
			m_response.setDateHeader( "Expires", lCurrent + lRecycle );
		}

		FileOutputStream	fileOutput = new FileOutputStream( strFilePath );

		m_outputStream.writeTo( fileOutput );
		fileOutput.close();

	}


    // SERVLET RESPONSE interface methods

    public void setContentLength( int len )
	{
		m_bSentContentLength = true;
		m_bCantCache = true;

		m_response.setContentLength( len );
    }

    public void setContentType( String type )
    {
		m_response.setContentType( type );
	}

    public ServletOutputStream getOutputStream()
	throws IOException, IllegalStateException
	{
		if( m_bGotOutput )
		{
			throw new IllegalStateException( "getOutputStream/getWriter has already be called" );
		}
		m_bGotOutput = true;

		return m_outputStream;
	}

    public String getCharacterEncoding()
	{
		return m_response.getCharacterEncoding();
	}

    public PrintWriter getWriter()
	throws UnsupportedEncodingException
    {
		if( m_bGotOutput )
		{
			throw new IllegalStateException( "getOutputStream/getWriter has already be called" );
		}
		m_bGotOutput = true;

		OutputStreamWriter	outputStreamWriter;
		outputStreamWriter = new OutputStreamWriter( m_outputStream, getCharacterEncoding() );
		m_printWriter = new PrintWriter( outputStreamWriter );

		return m_printWriter;
    }


    // HTTP SERVLET RESPONSE interface methods

    public void addCookie( Cookie cookie )
	{
		m_bCantCache = true;

		m_response.addCookie( cookie );
	}

    public boolean containsHeader( String name )
	{
		return m_response.containsHeader( name );
	}

    public void setStatus( int sc, String sm )
	{
		if( sc != HttpServletResponse.SC_OK )
		{
			m_bCantCache = true;
		}

		m_response.setStatus( sc, sm );
	}

    public void setStatus( int sc )
	{
		if( sc != HttpServletResponse.SC_OK )
		{
			m_bCantCache = true;
		}

		m_response.setStatus( sc );
	}

    public void setHeader( String name, String value )
	{
		m_bCantCache = true;

		m_response.setHeader( name, value );
	}

    public void setIntHeader( String name, int value )
	{
		m_bCantCache = true;

		m_response.setIntHeader( name, value );
	}

    public void setDateHeader( String name, long date )
	{
		m_bCantCache = true;

		m_response.setDateHeader( name, date );
	}

    public void sendError( int sc, String msg )
	throws IOException
	{
		m_bCantCache = true;

		m_response.sendError( sc, msg );
	}

    public void sendError( int sc )
	throws IOException
	{
		m_bCantCache = true;

		m_response.sendError( sc );
	}

    public void sendRedirect( String location )
	throws IOException
	{
		m_bCantCache = true;

		m_response.sendRedirect( location );
	}

    public String encodeUrl( String url ) 
	{
		return m_response.encodeUrl( url );
	}

    public String encodeRedirectUrl( String url )
	{
		return m_response.encodeRedirectUrl( url );
	}

}


/**
 *
 * Servlet output stream that buffers data.
 *
 */
class CacheOutputStream extends ServletOutputStream
{
	protected ByteArrayOutputStream	m_outputStream;

    // file private
    public CacheOutputStream()
	{
		m_outputStream = new ByteArrayOutputStream( 16 * 1024 );
	}

    public void write( int b )
	{
		m_outputStream.write( b );
    }

	public void writeTo( OutputStream out )
	throws IOException
	{
		m_outputStream.writeTo( out );
	}

	public int size()
	{
		return m_outputStream.size();
	}

}

